"""Module for constants in Counter-Strike 2."""

# Sides
CT_SIDE = "ct"
T_SIDE = "t"

# Server
DEFAULT_SERVER_TICKRATE = 128

# Rounds
DEFAULT_FREEZE_TIME_IN_SECS = 20.0
DEFAULT_ROUND_TIME_IN_SECS = 115.0
DEFAULT_BOMB_TIME_IN_SECS = 40.0

# Grenades
DEFAULT_SMOKE_DURATION_IN_SECS = 20.0
DEFAULT_INFERNO_DURATION_IN_SECS = 7.03125
