"""Converters for index-based fields."""

HITGROUP_MAP = {
    0: "generic",
    1: "head",
    2: "chest",
    3: "stomach",
    4: "left arm",
    5: "right arm",
    6: "left leg",
    7: "right leg",
    8: "neck",
    10: "gear",
}
